// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JFPPR. 
// 
// JFPPR is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JFPPR is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JFPPR.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.graph.fppr.demo;

public class Utils {
    public static int editDistance(String a, String b) {
        int i, j, I = a.length(), J = b.length();
        int[][] d = new int[I + 1][J + 1];
        for(i = 0;i <= I;i++)
            d[i][0] = i;
        for(j = 0;j <= J;j++)
            d[0][j] = j;
        for(j = 1;j <= J;j++)
            for(i = 1;i <= I;i++)
                if(a.charAt(i - 1) == b.charAt(j - 1))
                    d[i][j] = d[i - 1][j - 1];
                else
                    d[i][j] = Math.min(
                            d[i - 1][j] + 1, Math.min(
                            d[i][j - 1] + 1,
                            d[i - 1][j - 1] + 1));
        return d[I][J];
    }

    public static float scaledEditSimilarity(String a, String b) {
        if(a.isEmpty() || b.isEmpty())
            return 0;
        return 1.0f - (float)editDistance(a, b) / (float)Math.max(a.length(), b.length());
    }

    public static float scaledEditSimilarity(String a, String b, int max_edit_distance) {
        int edit_distance;
        if(a.isEmpty() || b.isEmpty() || (edit_distance = editDistance(a, b)) > max_edit_distance)
            return 0;
        return 1.0f - (float)edit_distance / (float)Math.max(a.length(), b.length());
    }
}
